#!/usr/bin/perl



use strict;

use Digest::SHA1;
use Time::HiRes;

require 'data.pl';
require 'string.pl';
require 'lock.pl';
require 'setting.pl';



my %argument = &string::splitQueryString( $ENV{'QUERY_STRING'} );
my $id = Digest::SHA1::sha1_base64( $ENV{'HTTP_USER_AGENT'} . $ENV{'REMOTE_HOST'} . $argument{'id'} );
$ENV{'TZ'} = $setting::setting{'timeZone'};
chdir( $setting::setting{'dataDirectory'} );



sub end
{
		print( "Content-type: text/plain; charset=UTF-8\n\n$_[0]" );
		exit;
}



# 入室
if ( $argument{'mode'} eq 'login' )
{
	# ユーザー名確認
	$argument{'name'}
		or &end( 'E入室処理失敗 - ユーザー名が指定されていません' );

	# ユーザー名変換（トリップ処理など）
	my $name = &string::trip( $argument{'name'} );

	# ロック
	my $lock;
	for (
		my $count = 0 ;
		!( $lock = new Lock( $setting::setting{'durationOfLock'} ) ) ;
		Time::HiRes::sleep( $setting::setting{'intervalOfRetryingLock'} ) )
	{
		++$count > $setting::setting{'maximalNumberOfRetryingLock'}
			and &end( 'S入室処理失敗 - 処理が混雑しています' );
	}

	#	ユーザーリスト読み込み
	my $userList = &data::loadUserList()
		or &end( 'S入室処理失敗 - ユーザーリストが読み込めません' );

	#	時間切れユーザー削除
	&data::removeTimeOutUser( $userList , $setting::setting{'limitOfTimeOut'} );

	#	ユーザーID確認
	my $currentUser = 0;
	foreach ( @$userList )
	{
		if ( $_->{'id'} eq $id )
		{
			$currentUser = $_;
			last;
		}
	}
	$currentUser or &end( 'R入室処理失敗 - ユーザーIDが不整合です' );

	# ◆切断済みは入室できない
	$currentUser->{'name'} eq '◆切断済み'
		and &end( 'R入室処理失敗 - 切断済みです' );

	# ◆閲覧者でなければ入室できない
	$currentUser->{'name'} eq '◆閲覧者'
		or &end( 'E入室処理失敗 - 既に入室しています' );

	# ユーザー名重複確認
	foreach ( @$userList )
	{
		$name eq $_->{'name'}
			and &end( 'E入室処理失敗 - ユーザー名が重複しています' );
	}

	# ユーザー名書き換え
	$currentUser->{'name'} = $name;

	# 更新時刻書き換え
	&data::saveLastUpdate()
		or &end( 'S入室処理失敗 - 更新時刻を書き込めません' );

	# 入室メッセージ書き込み
	&data::writeSystemMessage( $currentUser->{'name'} , $currentUser->{'userAgent'} , $currentUser->{'remoteHost'} , '入室' )
		or &end( 'S入室処理失敗 - 入室メッセージを書き込めません' );

	#	ユーザーリスト書き込み
	&data::saveUserList( $userList )
		or &end( 'S入室処理失敗 - ユーザーリストを書き込めません' );

	&end( 'O入室処理完了' );
}



# 退室
if ( $argument{'mode'} eq 'logout' )
{
	# ロック
	my $lock;
	for (
		my $count = 0 ;
		!( $lock = new Lock( $setting::setting{'durationOfLock'} ) ) ;
		Time::HiRes::sleep( $setting::setting{'intervalOfRetryingLock'} ) )
	{
		++$count > $setting::setting{'maximalNumberOfRetryingLock'}
			and &end( 'S退室処理失敗 - 処理が混雑しています' );
	}

	#	ユーザーリスト読み込み
	my $userList;
	$userList = &data::loadUserList()
		or &end( 'S退室処理失敗 - ユーザーリストが読み込めません' );

	#	時間切れユーザー削除
	&data::removeTimeOutUser( $userList , $setting::setting{'limitOfTimeOut'} );

	#	ユーザーID確認
	my $currentUser = 0;
	foreach ( @$userList )
	{
		if ( $_->{'id'} eq $id )
		{
			$currentUser = $_;
			last;
		}
	}
	$currentUser or &end( 'R退室処理失敗 - ユーザーIDが不整合です' );

	# ◆切断済みは退室できない
	$currentUser->{'name'} eq '◆切断済み'
		and &end( 'R退室処理失敗 - 切断済みです' );

	# ◆閲覧者は退室できない
	$currentUser->{'name'} eq '◆閲覧者'
		and &end( 'E退室処理失敗 - まだ入室していません' );

	# 更新時刻書き換え
	&data::saveLastUpdate()
		or &end( 'S退室処理失敗 - 更新時刻を書き込めません' );

	# 退室メッセージ書き込み
	&data::writeSystemMessage( $currentUser->{'name'} , $currentUser->{'userAgent'} , $currentUser->{'remoteHost'} , '退室' )
		or &end( 'S退室処理失敗 - 退室メッセージを書き込めません' );

	# ユーザー名を◆閲覧者に
	$currentUser->{'name'} = '◆閲覧者';

	#	ユーザーリスト書き込み
	&data::saveUserList( $userList )
		or &end( 'S退室処理失敗 - ユーザーリストを書き込めません' );

	&end( 'O退室処理完了' );
}



# 更新
if ( $argument{'mode'} eq 'reload' )
{
	for (
		my $startedAt = time() ;
		$startedAt + $setting::setting{'maximalTimeOfConnection'} > time() ;
		Time::HiRes::sleep( $setting::setting{'intervalOfCheckingAlteration'} ) )
	{
		{
			my $now = Time::HiRes::time();

			# ロック
			my $lock = new Lock( $setting::setting{'durationOfLock'} )
				or next;

			#	ユーザーリスト読み込み
			my $userList;
			$userList = &data::loadUserList()
				or &end( 'S更新処理失敗 - ユーザーリストが読み込めません' );

			#	時間切れユーザー削除
			my $isUserListModified
				= &data::removeTimeOutUser( $userList , $setting::setting{'limitOfTimeOut'} );

			#	ユーザーID確認
			my $currentUser = 0;
			foreach ( @$userList )
			{
				if ( $_->{'id'} eq $id )
				{
					$currentUser = $_;
					last;
				}
			}
			if ( !$currentUser )
			{
				$currentUser = {
					'id' => $id ,
					'name' => '◆閲覧者' ,
					'lastAccess' => 0 ,
					'userAgent' => $ENV{'HTTP_USER_AGENT'} ,
					'remoteHost' => $ENV{'REMOTE_HOST'} };
				push( @$userList , $currentUser );
				$isUserListModified = 1;
			}

			# ◆切断済みは閲覧できない
			$currentUser->{'name'} eq '◆切断済み'
				and &end( 'E更新処理失敗 - 切断済みです' );

			# 更新時刻書き換え
			if ( $isUserListModified )
			{
				&data::saveLastUpdate( $now )
					or &end( 'S更新処理失敗 - 更新時刻を書き込めません' );
			}

			# 更新があれば送信
			if ( &data::loadLastUpdate() > $currentUser->{'lastAccess'} )
			{
				&data::sendData( $userList , $currentUser->{'name'} , $setting::setting{'maximalNumberOfSendingMessage'} );
				$currentUser->{'lastAccess'} = $now;
				&data::saveUserList( $userList );
				exit;
			}

			#	最終更新確認時刻書き換え
			$currentUser->{'lastAccess'} = $now;

			#	ユーザーリスト書き込み
			&data::saveUserList( $userList )
				or &end( 'S更新処理失敗 - ユーザーリストを書き込めません' );
		}
	}
	&end( 'O更新処理完了 - 変更なし' );
}



# 投稿
if ( $argument{'mode'} eq 'post' )
{
	# メッセージ確認
	$argument{'statement'}
		or &end( 'E投稿処理失敗 - 文章が入力されていません' );

	# ロック
	my $lock;
	for (
		my $count = 0 ;
		!( $lock = new Lock( $setting::setting{'durationOfLock'} ) ) ;
		Time::HiRes::sleep( $setting::setting{'intervalOfRetryingLock'} ) )
	{
		++$count > $setting::setting{'maximalNumberOfRetryingLock'}
			and &end( 'S投稿処理失敗 - 処理が混雑しています' );
	}

	#	ユーザーリスト読み込み
	my $userList;
	$userList = &data::loadUserList()
		or &end( 'S投稿処理失敗 - ユーザーリストが読み込めません' );

	#	時間切れユーザー削除
	if ( &data::removeTimeOutUser( $userList , $setting::setting{'limitOfTimeOut'} ) )
	{
		&data::saveUserList( $userList )
			or &end( 'S投稿処理失敗 - ユーザーリストを書き込めません' );
	}

	#	ユーザーID確認
	my $currentUser = 0;
	foreach ( @$userList )
	{
		if ( $_->{'id'} eq $id )
		{
			$currentUser = $_;
			last;
		}
	}
	$currentUser or &end( 'R投稿処理失敗 - ユーザーIDが不整合です' );

	# ◆切断済みは投稿できない
	$currentUser->{'name'} eq '◆切断済み'
		and &end( 'R投稿処理失敗 - 切断済みです' );

	# ◆閲覧者は投稿できない
	$currentUser->{'name'} eq '◆閲覧者'
		and &end( 'E投稿処理失敗 - まだ入室していません' );

	# 更新時刻書き換え
	&data::saveLastUpdate()
		or &end( 'S投稿処理失敗 - 更新時刻を書き込めません' );

	# メッセージ書き込み
	&data::writeMessage( $currentUser->{'name'} , $currentUser->{'userAgent'} , $currentUser->{'remoteHost'} , $argument{'statement'} )
		or &end( 'S投稿処理失敗 - メッセージを書き込めません' );

	&end( 'O投稿処理完了' );
}



# 切断
if ( $argument{'mode'} eq 'disconnect' )
{
	# ロック
	my $lock;
	for (
		my $count = 0 ;
		!( $lock = new Lock( $setting::setting{'durationOfLock'} ) ) ;
		Time::HiRes::sleep( $setting::setting{'intervalOfRetryingLock'} ) )
	{
		++$count > $setting::setting{'maximalNumberOfRetryingLock'}
			and &end( 'S切断処理失敗 - 処理が混雑しています' );
	}

	#	ユーザーリスト読み込み
	my $userList;
	$userList = &data::loadUserList()
		or &end( 'S切断処理失敗 - ユーザーリストが読み込めません' );

	#	時間切れユーザー削除
	&data::removeTimeOutUser( $userList , $setting::setting{'limitOfTimeOut'} );

	#	ユーザーID確認
	my $currentUser = 0;
	foreach ( @$userList )
	{
		if ( $_->{'id'} eq $id )
		{
			$currentUser = $_;
			last;
		}
	}
	$currentUser or &end( 'R切断処理失敗 - ユーザーIDが不整合です' );

	# ◆切断済みは切断できない
	$currentUser->{'name'} eq '◆切断済み'
		and &end( 'R切断処理失敗 - 切断済みです' );

	# 更新時刻書き換え
	&data::saveLastUpdate()
		or &end( 'S切断処理失敗 - 更新時刻を書き込めません' );

	# ◆閲覧者でなければ退室メッセージ書き込み
	if ( $currentUser->{'name'} ne '◆閲覧者' )
	{
		&data::writeSystemMessage( $currentUser->[1] , $currentUser->[3] , $currentUser->[4] , '退室' )
			or &end( 'S切断処理失敗 - 退室メッセージを書き込めません' );
	}

	# ユーザー名を◆切断済みに
	$currentUser->{'name'} = '◆切断済み';

	#	ユーザーリスト書き込み
	&data::saveUserList( $userList )
		or &end( 'S切断処理失敗 - ユーザーリストを書き込めません' );

	&end( 'O切断処理完了' );
}

&end( "パラメータが不正です" );
