
use strict;

use Time::HiRes;


package data;



#	返り値	成功なら最終更新時刻，失敗なら0
sub loadLastUpdate
{
	open( FILE , '< lastupdate' ) or return 0;
	my $lastUpdate = <FILE>;
	close(FILE);
	return 0 + $lastUpdate;
}



#	引数 1	時刻（省略で現在時刻）
#	返り値	成功なら1，失敗なら0
sub saveLastUpdate
{
	open( FILE , '> lastupdate' ) or return 0;
	print FILE ( $_[0] ? $_[0] : Time::HiRes::time() );
	close(FILE);
	return 1;
}



# 引数 1	発言者
#	引数 2	ユーザーエージェント
#	引数 3	リモートホスト
#	引数 4	内容
#	引数 5	時刻（省略で現在時刻）
#	返り値	成功なら1，失敗なら0
sub writeMessage
{
	open( FILE , '>> conversation' ) or return 0;
	my ( $sec , $min , $hour , $mday , $mon , $year , $wday , $yday , $isdst ) = localtime( $_[4] ? int( $_[4] ) : time() );
	my $timeString = sprintf( '%d/%02d/%02d %02d:%02d:%02d' , $year + 1900 , $mon + 1 , $mday , $hour , $min , $sec , $hour , $min , $sec );
	print FILE "<tr><td class=\"speaker\" title=\"UA: $_[1] , Host: $_[2]\">$_[0]</td><td class=\"statement\">$_[3]</td><td class=\"posted_at\">$timeString</td></tr>\n";
	close(FILE);
	return 1;
}



# 引数 1	発言者
#	引数 2	ユーザーエージェント
#	引数 3	リモートホスト
#	引数 4	内容
#	引数 5	時刻（省略で現在時刻）
#	返り値	成功なら1，失敗なら0
sub writeSystemMessage
{
	open( FILE , '>> conversation' ) or return 0;
	my ( $sec , $min , $hour , $mday , $mon , $year , $wday , $yday , $isdst ) = localtime( $_[4] ? int( $_[4] ) : time() );
	my $timeString = sprintf( '%d/%02d/%02d %02d:%02d:%02d' , $year + 1900 , $mon + 1 , $mday , $hour , $min , $sec , $hour , $min , $sec );
	print FILE "<tr class=\"system_message\"><td class=\"speaker\" title=\"UA: $_[1] , Host: $_[2]\">$_[0]</td><td class=\"statement\">$_[3]</td><td class=\"posted_at\">$timeString</td></tr>\n";
	close(FILE);
	return 1;
}



#	返り値	成功ならユーザーリストへのリファレンス，失敗なら0
sub loadUserList
{
	open( IN , '< users' ) or return 0;
	my @users;
	while ( my $line = <IN> )
	{
		chomp( $line );
		$line or next;
		my @newUser = split( '<>' , $line );
		push( @users , {
			'id' => $newUser[0] ,
			'name' => $newUser[1] ,
			'lastAccess' => 0 + $newUser[2] ,
			'userAgent' => $newUser[3] ,
			'remoteHost' => $newUser[4] } );
	}
	close(IN);
	return \@users;
}



#	引数 1	ユーザーリストへのリファレンス
#	返り値	成功なら1，失敗なら0
sub saveUserList
{
	$_[0] or return 0;
	open( OUT , '> users' ) or return 0;
	foreach ( @{$_[0]} ) { print OUT "$_->{'id'}<>$_->{'name'}<>$_->{'lastAccess'}<>$_->{'userAgent'}<>$_->{'remoteHost'}\n"; }
	close(OUT);
	return 1;
}



#	引数 1	ユーザーリストへのリファレンス
#	引数 2	時間切れ猶予
#	返り値	変更があれば1，なければ0
sub removeTimeOutUser
{
	$_[0] or return 0;
	my $now = Time::HiRes::time();
	my $isModified = 0;
	for ( my $i = 0 ; $i < @{$_[0]} ; ++$i )
	{
		#	時間切れ
		if ( $_[0]->[$i]->{'lastAccess'} + $_[1] < $now )
		{
			if ( $_[0]->[$i]->{'name'} ne '◆切断済み' )
			{
				# 入室者なら時間切れメッセージを書き込み
				$_[0]->[$i]->{'name'} ne '◆閲覧者'
					and &writeSystemMessage(
						$_[0]->[$i]->{'name'} ,
						$_[0]->[$i]->{'userAgent'} ,
						$_[0]->[$i]->{'remoteHost'} ,
						'接続期限切れ退室' ,
						$_[0]->[$i]->{'lastAccess'} + $_[1] );
				$isModified = 1;
			}
			#	要素を削除
			splice( @{$_[0]} , $i , 1 );
			--$i;
		}
	}
	return $isModified;
}



#	引数 1	会話リストの最大サイズ（バイト単位）
#	返り値	成功なら1，失敗なら0
sub moveConversationToLog
{
	#	現在のファイルサイズが指定サイズよりも小さかったら何もしない．
	my $size = -s 'conversation';
	if ( $size < $_[0] ) { return 1; }

	#	会話リスト読み込み
	my $conversation;
	open( IN , '< conversation' ) or return 0;
	read( IN , $conversation , $size );
	close(IN);

	my $cutLine = index( $conversation , "\n" , $size - $_[0] ) + 1;

	#	ログファイルに追記
	open( LOG , '>> log' ) or return 0;
	syswrite( LOG , $conversation , $cutLine );
	close(LOG);

	#	会話リスト書き込み
	open( OUT , '> conversation' ) or return 0;
	syswrite( OUT , $conversation , $size - $cutLine , $cutLine );
	close(OUT);

	return 1;
}



# データをブラウザに送信します
#	引数 1	ユーザーリスト
# 引数 2  ユーザー名
#	引数 3	出力する会話行数
#	返り値	成功なら1，失敗なら0
sub sendData
{
	#	引数チェック
	( $_[0] and $_[1] ) or return 0;

	#	会話データ読み込み
	open( IN , '< conversation' ) or return 0;
	my @conversation = <IN>;
	close(IN);

	#	HTTPヘッダ出力
	print( "Content-type: text/plain; charset=UTF-8\n\n" );

	# ユーザー名出力
	print( "D$_[1]<>\n" );

	my $rom = 0;

	# ユーザーリスト出力
	foreach ( @{$_[0]} )
	{
		# 閲覧者カウント
		if ( $_->{'name'} eq '◆閲覧者' ) { ++$rom; }

		# 切断済みでなければ表示
		elsif ( $_->{'name'} ne '◆切断済み' )
		{
			print( "<li title=\"UA: $_->{'userAgent'} , Host: $_->{'remoteHost'}\">$_->{'name'}</li>\n" );
		}
	}
	print( "<li class=\"rom\">◆閲覧者 $rom 人</li>\n<>\n<table><tbody>\n" );


	# 会話があれば出力
	if ( @conversation )
	{
		#	読み出し開始位置，終了位置を確定
		my $i = @conversation;
		my $end = $i - $_[2];
		$end < 0 and $end = 0;

		# 最近の投稿が上になるように，逆順で出力
		while ( --$i >= $end ) { print( $conversation[$i] ); }
	}

	# 会話がなければ空セルを出力
	else
	{
		print( "<tr><td class=\"speaker\"></td><td class=\"statement\"></td><td class=\"posted_at\"></td></tr>\n" );
	}

	print( "</tbody></table>\n" );

	return 1;
}



1;
